'''
Ice Palace
'''

__all__ = 'LOCATIONS',


LOCATIONS = {
    'Ice Palace Entrance (E)': {
        'type': 'entrance_dungeon', 'map': 'dark', 'coord': (523, 573),
        'link': {
            'Ice Palace Exterior': [],
            'Ice Palace Entrance (I)': []}
    },
    'Ice Palace Entrance (I)': {
        'type': 'interior',
        'link': {
            'Ice Palace Entrance (E)': [],
            'Ice Palace Floor 1 Room 1': []}
    },
    'Ice Palace Floor 1 Room 1': {
        'type': 'area', 'dungeon': 'Ice Palace',
        'link': {
            'Ice Palace Entrance (I)': [],
            'Ice Palace Floor 1 Room 2': [
                ('item', 'firerod'),
                ('and', [
                    ('item', 'bombos'),
                    ('or', [
                        ('settings', 'swordless'), ('item', 'sword')])])]}
    },
    'Ice Palace Floor 1 Room 2': {
        'type': 'area', 'dungeon': 'Ice Palace',
        'link': {
            'Ice Palace Floor 1 Room 1': [],
            'Ice Palace First Bari Key': [],
            'Ice Palace Floor 2': [('smallkey', 'Ice Palace')]}
    },
    'Ice Palace First Bari Key': {
        'type': 'dungeonkey', 'dungeon': 'Ice Palace',
        'link': {
            'Ice Palace Floor 1 Room 2': []}
    },
    'Ice Palace Floor 2': {
        'type': 'area', 'dungeon': 'Ice Palace',
        'link': {
            'Ice Palace Floor 1 Room 2': [],
            'Ice Palace Compass': [],
            'Ice Palace Floor 3': [('item', 'bombs')]}
    },
    'Ice Palace Compass': {
        'type': 'dungeonchest', 'dungeon': 'Ice Palace',
        'link': {
            'Ice Palace Floor 2': []}
    },
    'Ice Palace Floor 3': {
        'type': 'area', 'dungeon': 'Ice Palace',
        'link': {
            'Ice Palace Floor 2': [],
            'Ice Palace Second Bari Key': [],
            'Ice Palace Floor 4': [('smallkey', 'Ice Palace')]}
    },
    'Ice Palace Second Bari Key': {
        'type': 'dungeonkey', 'dungeon': 'Ice Palace',
        'link': {
            'Ice Palace Floor 3': []}
    },
    'Ice Palace Floor 4': {
        'type': 'area', 'dungeon': 'Ice Palace',
        'link': {
            'Ice Palace Floor 3': [],
            'Ice Palace Lower Ascent': [('smallkey', 'Ice Palace')],
            'Ice Palace Big Ice Floor': [],
            'Ice Palace Freezor Room': []}
    },
    'Ice Palace Lower Ascent': {
        'type': 'area', 'dungeon': 'Ice Palace',
        'link': {
            'Ice Palace Floor 4': [],
            'Ice Palace Ascent Key': [],
            'Ice Palace Map': [],
            'Ice Palace Upper Ascent': [('and', [
                ('item', 'hammer'), ('item', 'powerglove'),
                ('item', 'bombs')])]}
    },
    'Ice Palace Ascent Key': {
        'type': 'dungeonchest', 'dungeon': 'Ice Palace',
        'link': {
            'Ice Palace Lower Ascent': []}
    },
    'Ice Palace Map': {
        'type': 'dungeonchest', 'dungeon': 'Ice Palace',
        'link': {
            'Ice Palace Lower Ascent': []}
    },
    'Ice Palace Upper Ascent': {
        'type': 'area', 'dungeon': 'Ice Palace',
        'link': {
            'Ice Palace Lower Ascent': [],
            'Ice Palace Upper Hidden Key': [],
            # Generally speaking, the bottom key can be reached without using a
            # small key via. But this is not what we're after, therefore this
            # 'virtual keydoor' is placed here.
            'Ice Palace Big Key': [('smallkey', 'Ice Palace')],
            'Ice Palace Floor 2': []}
    },
    'Ice Palace Upper Hidden Key': {
        'type': 'dungeonkey', 'dungeon': 'Ice Palace',
        'link': {
            'Ice Palace Upper Ascent': []}
    },
    'Ice Palace Big Key': {
        'type': 'dungeonchest', 'dungeon': 'Ice Palace',
        'link': {
            'Ice Palace Upper Ascent': []}
    },
    'Ice Palace Big Ice Floor': {
        'type': 'area', 'dungeon': 'Ice Palace',
        'link': {
            'Ice Palace Lower Ascent': [('item', 'hookshot')],
            'Ice Palace Bottom Key': [('smallkey', 'Ice Palace')],
            'Ice Palace Boss Puzzle': []},
    },
    'Ice Palace Freezor Room': {
        'type': 'area', 'dungeon': 'Ice Palace',
        'link': {
            'Ice Palace Floor 4': [],
            'Ice Palace Freezor Chest': [
                ('item', 'firerod'),
                ('and', [
                    ('item', 'bombos'),
                    ('or', [
                        ('item', 'sword'), ('settings', 'swordless')])])],
            'Ice Palace Treasure Drop': [('item', 'bombs')],
            'Ice Palace Boss Puzzle': []}
    },
    'Ice Palace Freezor Chest': {
        'type': 'dungeonchest', 'dungeon': 'Ice Palace',
        'link': {
            'Ice Palace Freezor Room': []}
    },
    'Ice Palace Treasure Drop': {
        'type': 'area', 'dungeon': 'Ice Palace',
        'link': {
            'Ice Palace Treasure': [('bigkey', 'Ice Palace')],
            'Ice Palace Boss Puzzle': []}
    },
    'Ice Palace Treasure': {
        'type': 'dungeonchest_nokey', 'dungeon': 'Ice Palace',
        'link': {
            'Ice Palace Treasure Drop': []}
    },
    'Ice Palace Boss Puzzle': {
        'type': 'area', 'dungeon': 'Ice Palace',
        'link': {
            'Ice Palace Boss Puzzle Solution': [('bigkey', 'Ice Palace')],
            'Ice Palace Babusu Room': []}
    },
    'Ice Palace Babusu Room': {
        'type': 'area', 'dungeon': 'Ice Palace',
        'link': {
            'Ice Palace Boss Puzzle': [],
            'Ice Palace Lower Hidden Key': [],
            'Ice Palace Ice Bridge': [('smallkey', 'Ice Palace')]}
    },
    'Ice Palace Lower Hidden Key': {
        'type': 'dungeonkey', 'dungeon': 'Ice Palace',
        'link': {
            'Ice Palace Babusu Room': []}
    },
    'Ice Palace Ice Bridge': {
        'type': 'area', 'dungeon': 'Ice Palace',
        'link': {
            'Ice Palace Babusu Room': [],
            # Another 'virtual door'. Ice Palace sure is special.
            'Ice Palace Bottom Key': [('smallkey', 'Ice Palace')]}
    },
    'Ice Palace Bottom Key': {
        'type': 'dungeonchest', 'dungeon': 'Ice Palace',
        'link': {
            'Ice Palace Ice Bridge': [],
            'Ice Palace Big Ice Floor': []}
    },
    'Ice Palace Boss Puzzle Solution': {
        'type': 'area', 'dungeon': 'Ice Palace',
        'link': {
            'Ice Palace Above Boss': [('smallkey', 'Ice Palace')],
            'Ice Palace Somaria Puzzle': [
                ('settings', 'placement_basic'), ('item', 'somaria')],
            'Ice Palace Boss Puzzle': []}
    },
    'Ice Palace Somaria Puzzle': {
        'type': 'area', 'dungeon': 'Ice Palace',
        'link': {
            'Ice Palace Boss Puzzle Solution': [],
            'Ice Palace Above Boss': []}
    },
    'Ice Palace Above Boss': {
        'type': 'area', 'dungeon': 'Ice Palace',
        'link': {
            'Ice Palace Boss Puzzle Solution': [],
            'Ice Palace Boss': [('and', [
                ('item', 'hammer'), ('item', 'powerglove')])]}
    },
    'Ice Palace Boss': {
        'type': 'dungeonboss', 'dungeon': 'Ice Palace',
        'link': {
            'Ice Palace Boss Item': [('and', [
                ('or', [
                    ('item', 'firerod'),
                    ('and', [
                        ('item', 'bombos'),
                        ('or', [
                            ('settings', 'swordless'), ('item', 'sword')])])]),
                ('or', [
                    ('item', 'sword'), ('item', 'hammer'),
                    ('and', [
                        ('or', [
                            ('item', 'bottle'), ('item', 'halfmagic')]),
                        ('or', [
                            ('item', 'firerod'), ('item', 'bombos')])])]),
                ('or', [
                    ('settings', 'placement_advanced'),
                    ('item', 'mastersword'),
                    ('and', [
                        ('or', [
                            ('item', 'bottle'), ('item', 'halfmagic')]),
                        ('item', 'firerod')])])])]}
    },
    'Ice Palace Boss Item': {
        'type': 'dungeonchest_nokey', 'dungeon': 'Ice Palace',
        'link': {
            'Ice Palace Reward': []}
    },
    'Ice Palace Reward': {
        'type': 'dungeonreward', 'dungeon': 'Ice Palace',
        'link': {
            'Ice Palace Entrance (I)': []}
    },
}



'''
Program GUI
'''

import importlib
import os
import re
import threading
import tkinter as tk
import tkinter.filedialog as filedialog
import tkinter.messagebox as tkmbox
import tkinter.ttk as ttk
import typing

from ..config import CONFIGDIRECTORY, CONFIG
common = importlib.import_module(
    '..gui-common.interface', package=__package__)
from .. import update
from .. import version

from . import config
from . import dark
from . import dungeons
from . import environ
from . import help
from . import items
from . import light
from . import misc
from . import ttkstyle

__all__ = 'GraphicalInterface',


class GraphicalInterface(tk.Tk):
    '''
    Main access point for everything GUI related.

    Instance variables:
        _gui_root: tkinter framework root object
        _menu: main menu
        _buttons: buttons in menu
        _tracker: world state tracker
        _windows: collection of child windows
    '''

    def __init__(self):
        '''
        Initialise GUI.
        '''

        super().__init__()
        self.call('tk', 'scaling', 1)
        ttkstyle.init()
        self._init_menu()

        self._tracker = common.world_tracker()

        self._windows = {'menu': self}

        self._restore_windows()
        self._prepare_windows()
        self.protocol('WM_DELETE_WINDOW', self.quit)

        self._start_update_check()

    def _restore_windows(self) -> None:
        '''
        Restore previously stored window layout.
        '''

        layout = common.load_window_cache()
        for window in layout:
            if window not in self._windows and window in environ.WINDOWLIST:
                self._start_window(window)
            if window in self._windows:
                self._windows[window].geometry(
                    '+{0:d}+{1:d}'.format(*layout[window]))

    def _start_window(self, identifier: str) -> None:
        '''
        Initialise a window.

        Args:
            identifier: name of window to open
        '''

        if identifier == 'quit':
            self.quit()
            return
        if identifier == 'reset':
            self.reset()
            return
        if identifier == 'save':
            self.save()
            return
        if identifier == 'load':
            self.load()
            return
        if identifier in self._windows:
            self._windows[identifier].deiconify()
            return
        self._windows[identifier] = globals()[identifier].start(self._tracker)
        if identifier == 'config':
            self._windows[identifier].dungeondisplay = self._windows['dungeons']
        self._windows[identifier].protocol(
            'WM_DELETE_WINDOW', self._windows[identifier].withdraw)

    def run(self) -> None:
        '''
        Run main GUI loop.
        '''

        self.mainloop()

    def quit(self) -> None:
        '''
        Quit program.
        '''

        common.save_window_cache(self._window_layout())
        for window in self._windows:
            if window != 'menu':
                self._windows[window].withdraw()
        self._menu.quit()

    def _prepare_windows(self) -> None:
        '''
        Preload windows without displaying them.

        I don't really want to deal with the hassle of non-existing
        windows/trackers, so I do this.
        '''

        prepwindows = []
        for window in environ.WINDOWLIST:
            if window not in self._windows:
                prepwindows.append(window)
        for window in prepwindows:
            self._start_window(window)
        for window in prepwindows:
            self._windows[window].withdraw()

    def _open_window(self, window: str) -> None:
        '''
        Open a window.

        Args:
            window: name of existing window object
            creator: window creation routine
        '''

        try:
            self._windows[window].deiconify()
        except (AttributeError, tk.TclError):
            self._start_window(window)
        self._windows[window].protocol(
            'WM_DELETE_WINDOW', self._windows[window].withdraw)

    def _window_layout(self) -> dict:
        '''
        Return current position of all windows.

        Returns:
            dict: {window name: (x, y)}
        '''

        layout = {}
        for window in self._windows:
            if self._windows[window].state() == 'withdrawn':
                continue
            try:
                self._windows[window].deiconify()
            except (AttributeError, tk.TclError):
                continue
            layout[window] = tuple(
                int(c) for c in re.match(
                    '(\d+)x(\d+)\+-?(\d+)\+-?(\d+)',
                    self._windows[window].geometry()).groups()[2:])
        return layout

    def _init_menu(self) -> None:
        '''
        Create menu buttons.
        '''

        self.title('Menu')

        self._menu = ttk.Frame(self)
        self._menu.grid(column=0, row=0, sticky=misc.A)

        self._buttons = {
            'items': self._make_button('items', (0, 0), 'Items'),
            'dungeons': self._make_button('dungeons', (1, 0), 'Dungeons'),
            'light': self._make_button('light', (0, 1), 'Light World'),
            'dark': self._make_button('dark', (1, 1), 'Dark World'),
            'config': self._make_button('config', (0, 3), 'Settings'),
            'help': self._make_button('help', (1, 3), 'Help'),
            'save': self._make_button('save', (0, 4), 'Save'),
            'load': self._make_button('load', (1, 4), 'Load'),
            'reset': self._make_button('reset', (0, 5), 'Reset'),
            'quit': self._make_button('quit', (1, 5), 'Quit'),
            }

    def _make_button(
            self, identifier: str, loc: typing.Sequence[int],
            text: str or tk.StringVar) -> ttk.Button:
        '''
        Shortcut to place buttons.

        Args:
            identifier: name of button
            loc: (column, row) of button on 2D grid
            text: text to display on button

        Returns:
            ttk.Button: created button
        '''

        button = ttk.Button(
            self._menu, command=lambda: self._start_window(identifier))
        if isinstance(text, tk.StringVar):
            button.configure(textvariable=text)
        else:
            assert isinstance(text, str)
            button.configure(text=text)
        button.grid(column=loc[0], row=loc[1], sticky=tk.N+tk.W+tk.E)
        return button

    def reset(self) -> None:
        '''
        Reset tracker windows.
        '''

        check = tkmbox.askokcancel(
            'Reset', 'This will delete all stored progress.',
            default=tkmbox.CANCEL, icon=tkmbox.WARNING)
        if not check:
            return

        for window in self._windows:
            if window not in ('menu', 'config'):
                self._windows[window].reset()

        try:
            os.remove(os.path.join(CONFIGDIRECTORY, CONFIG['autosave']))
        except FileNotFoundError:
            pass

    def save(self) -> None:
        '''
        Save tracker state to file.
        '''

        savefile = filedialog.asksaveasfilename(
            defaultextension='.json', filetypes=(('JSON', '*.json'),),
            parent=self, title='Save to file')
        if not savefile:
            return
        with open(
                os.path.join(CONFIGDIRECTORY, CONFIG['autosave']), 'r') as src:
            with open(savefile, 'w') as dst:
                dst.write(src.read())

    def load(self) -> None:
        '''
        Load save file.
        '''

        loadfile = filedialog.askopenfilename(
            defaultextension='.json', filetypes=(('JSON', '*.json'),),
            parent=self, title='Load file')
        if not loadfile:
            return
        with open(
                os.path.join(CONFIGDIRECTORY, CONFIG['autosave']), 'w') as dst:
            with open(loadfile, 'r') as src:
                dst.write(src.read())
        tkmbox.showinfo('Load file', 'Please restart z3-tracker.')
        self.quit()

    def _start_update_check(self) -> None:
        '''
        Initiate update check.
        '''

        check = threading.Thread(
            target=self._update_check, name='Update checker', daemon=True)
        check.start()

    def _update_check(self) -> None:
        '''
        Check for updates.
        '''

        update_available = update.check_update_availability()
        if update_available:
            update_text='''Update available:
- current: {0:s}
- newest: {1:s}'''.format(version.__version__, update_available)
            update_frame = ttk.Frame(self)
            update_frame.grid(column=0, row=1, sticky=misc.A)
            update_label = ttk.Label(
                update_frame, text=update_text)
            update_label.grid(sticky=misc.A)



'''
Kakariko
'''

__all__ = 'LOCATIONS',


LOCATIONS = {
    'Kakariko': {
        'type': 'area',
        'link': {
            "Thieves' Town Surface": [('and', [
                ('settings', 'inverted'), ('item', 'mirror')])],
            "East Thieves' Town": [('and', [
                ('settings', 'inverted'), ('item', 'mirror')])],
            'Light Bottle Ocarina': [],
            "Blacksmiths' House Entrance (E)": [],
            'Above The Bat': [
                ('item', 'hammer'),
                ('and', [
                    ('glitch', 'overworld'), ('item', 'pegasus')])],
            "Mad Batter's Cave Entrance Entrance (E)": [],
            'South Light World': [],
            'Tavern Front Entrance (E)': [],
            'Kakariko Shop Entrance (E)': [],
            'Secret House Entrance (E)': [('item', 'bombs')],
            'Cucco House Entrance (E)': [],
            "Bug Child's House Entrance (E)": [],
            'Tavern Back Entrance': [('rabbitbarrier', None)],
            'Kakariko Blue House Entrance (E)': [('rabbitbarrier', None)],
            'Kakariko East Red House Entrance (E)': [],
            'Bottle Vendor': [],
            'Kakariko West Red House Entrance (E)': [],
            'Kakariko Cave Entrance Entrance (E)': [],
            'Kakariko Cave Drop Entrance (E)': [],
            'Lost Woods': [('rabbitbarrier', None)],
            'Kakariko Portal (LW)': [('item', 'titansmitts')],
            "Blind's House Entrance (E)": [],
            "Sahasrahla's House West Entrance Entrance (E)": [],
            "Sahasrahla's House East Entrance Entrance (E)": [],
            'North Light World': []}
    },
    "Blacksmiths' House Entrance (E)": {
        'type': 'entrance_unique', 'map': 'light', 'coord': (202, 353),
        'link': {
            'Kakariko': [],
            "Blacksmiths' House Entrance (I)": []}
    },
    "Blacksmiths' House Entrance (I)": {
        'type': 'interior',
        'link': {
            "Blacksmiths' House Entrance (E)": [],
            'Blacksmiths': [('and', [
                ('access', 'Frog Prison'),
                ('or', [
                    ('settings', 'placement_advanced'), ('item', 'mirror')])])]}
    },
    'Blacksmiths': {
        'type': 'chest', 'map': 'light', 'coord': (202, 347),
        'link': {
            "Blacksmiths' House Entrance (I)": []}
    },
    'Above The Bat': {
        'type': 'area',
        'link': {
            'Kakariko': [('item', 'hammer')],
            "Mad Batter's Cave Drop Entrance (E)": []}
    },
    "Mad Batter's Cave Drop Entrance (E)": {
        'type': 'entrance_drop', 'map': 'light', 'coord': (224, 365),
        'link': {
            "Mad Batter's Cave Drop Entrance (I)": []}
    },
    "Mad Batter's Cave Drop Entrance (I)": {
        'type': 'interior',
        'link': {
            "Mad Batter's Cave": []}
    },
    "Mad Batter's Cave": {
        'type': 'area',
        'link': {
            'Mad Batter': [('item', 'powder')],
            "Mad Batter's Cave Front": []}
    },
    'Mad Batter': {
        'type': 'drop', 'map': 'light', 'coord': (215, 373),
        'link': {
            "Mad Batter's Cave": []}
    },
    "Mad Batter's Cave Front": {
        'type': 'area',
        'link': {
            "Mad Batter's Cave Entrance Entrance (I)": []}
    },
    "Mad Batter's Cave Entrance Entrance (I)": {
        'type': 'interior',
        'link': {
            "Mad Batter's Cave Front": [],
            "Mad Batter's Cave Entrance Entrance (E)": []},
    },
    "Mad Batter's Cave Entrance Entrance (E)": {
        'type': 'entrance_unique', 'map': 'light', 'coord': (210, 368),
        'link': {
            "Mad Batter's Cave Entrance Entrance (I)": [],
            'Kakariko': []}
    },
    'Tavern Front Entrance (E)': {
        'type': 'entrance', 'map': 'light', 'coord': (106, 395),
        'link': {
            'Kakariko': [],
            'Tavern Front Entrance (I)': []}
    },
    'Tavern Front Entrance (I)': {
        'type': 'interior',
        'link': {
            'Tavern Front Entrance (E)': []}
    },
    'Kakariko Shop Entrance (E)': {
        'type': 'entrance_shop', 'map': 'light', 'coord': (73, 387),
        'link': {
            'Kakariko': [],
            'Kakariko Shop Entrance (I)': []}
    },
    'Kakariko Shop Entrance (I)': {
        'type': 'interior',
        'link': {
            'Kakariko Shop Entrance (E)': []}
    },
    'Secret House Entrance (E)': {
        'type': 'entrance', 'map': 'light', 'coord': (18, 394),
        'link': {
            'Kakariko': [],
            'Cucco House Entrance (I)': []}
    },
    'Secret House Entrance (I)': {
        'type': 'interior',
        'link': {
            'Secret House Entrance (E)': []}
    },
    'Cucco House Entrance (E)': {
        'type': 'entrance_unique', 'map': 'light', 'coord': (65, 358),
        'link': {
            'Kakariko': [],
            'Cucco House Entrance (E)': []}
    },
    'Cucco House Entrance (I)': {
        'type': 'interior',
        'link': {
            'Cucco House Entrance (E)': [],
            'Cucco House': [('item', 'bombs')]}
    },
    'Cucco House': {
        'type': 'chest', 'map': 'light', 'coord': (65, 350),
        'link': {
            'Cucco House Entrance (I)': []}
    },
    "Bug Child's House Entrance (E)": {
        'type': 'entrance_unique', 'map': 'light', 'coord': (104, 356),
        'link': {
            'Kakariko': [],
            "Bug Child's House Entrance (I)": []}
    },
    "Bug Child's House Entrance (I)": {
        'type': 'interior',
        'link': {
            "Bug Child's House Entrance (E)": [],
            'Bug Child': [('item', 'bottle')]}
    },
    'Bug Child': {
        'type': 'chest', 'map': 'light', 'coord': (104, 351),
        'link': {
            "Bug Child's House Entrance (I)": []}
    },
    'Tavern Back Entrance': {
        'type': 'item', 'map': 'light', 'coord': (106, 376),
        'link': {
            'Kakariko': []}
    },
    'Kakariko Blue House Entrance (E)': {
        'type': 'entrance', 'map': 'light', 'coord': (135, 353),
        'link': {
            'Kakariko': [],
            'Kakariko Blue House Entrance (I)': []}
    },
    'Kakariko Blue House Entrance (I)': {
        'type': 'interior',
        'link': {
            'Kakariko Blue House Entrance (E)': []}
    },
    'Kakariko East Red House Entrance (E)': {
        'type': 'entrance', 'map': 'light', 'coord': (137, 319),
        'link': {
            'Kakariko': [],
            'Kakariko East Red House Entrance (I)': []}
    },
    'Kakariko East Red House Entrance (I)': {
        'type': 'interior',
        'link': {
            'Kakariko East Red House Entrance (E)': []}
    },
    'Bottle Vendor': {
        'type': 'item', 'map': 'light', 'coord': (63, 308),
        'link': {
            'Kakariko': []}
    },
    'Kakariko West Red House Entrance (E)': {
        'type': 'entrance', 'map': 'light', 'coord': (33, 309),
        'link': {
            'Kakariko': [],
            'Kakariko West Red House Entrance (I)': []}
    },
    'Kakariko West Red House Entrance (I)': {
        'type': 'interior',
        'link': {
            'Kakariko West Red House Entrance (E)': []}
    },
    'Kakariko Cave Drop Entrance (E)': {
        'type': 'entrance_drop', 'map': 'light', 'coord': (15, 282),
        'link': {
            'Kakariko Cave Drop Entrance (I)': []}
    },
    'Kakariko Cave Drop Entrance (I)': {
        'type': 'interior',
        'link': {
            'Kakariko Cave': [('item', 'bombs')],
            'Kakariko Cave Bottom': []}
    },
    'Kakariko Cave': {
        'type': 'drop', 'map': 'light', 'coord': (15, 282),
        'link': {
            'Kakariko Cave Bottom': []}
    },
    'Kakariko Cave Bottom': {
        'type': 'area',
        'link': {
            'Kakariko Cave Entrance Entrance (I)': []}
    },
    'Kakariko Cave Entrance Entrance (I)': {
        'type': 'interior',
        'link': {
            'Kakariko Cave Bottom': [],
            'Kakariko Cave Entrance Entrance (E)': []}
    },
    'Kakariko Cave Entrance Entrance (E)': {
        'type': 'entrance_unique', 'map': 'light', 'coord': (31, 283),
        'link': {
            'Kakariko Cave Entrance Entrance (I)': [],
            'Kakariko': []}
    },
    'Kakariko Portal (LW)': {
        'type': 'area',
        'link': {
            'Kakariko Portal (DW)': [('and', [
                ('settings', 'inverted'), ('item', 'mirror')])],
            'Kakariko Portal': [('item', 'powerglove')],
            'Kakariko': [('item', 'titansmitts')],
            'Lost Woods': [('item', 'hammer')]}
    },
    'Kakariko Portal': {
        'type': 'area',
        'link': {
            'Kakariko Portal (LW)': [('and', [
                ('settings', 'inverted'), ('state', 'rabbit')])],
            'Kakariko Portal (DW)': [('and', [
                ('nosettings', 'inverted'), ('state', 'rabbit')])]}
    },
    "Blind's House Entrance (E)": {
        'type': 'entrance_unique', 'map': 'light', 'coord': (85, 279),
        'link': {
            'Kakariko': [],
            "Blind's House Entrance (I)": []}
    },
    "Blind's House Entrance (I)": {
        'type': 'interior',
        'link': {
            "Blind's House Entrance (E)": [],
            "Blind's House": [('item', 'bombs')]}
    },
    "Blind's House": {
        'type': 'chest', 'map': 'light', 'coord': (85, 272),
        'link': {
            "Blind's House Entrance (I)": []}
    },
    "Sahasrahla's House East Entrance Entrance (E)": {
        'type': 'entrance_unique', 'map': 'light', 'coord': (115, 279),
        'link': {
            'Kakariko': [],
            "Sahasrahla's House East Entrance Entrance (I)": []}
    },
    "Sahasrahla's House East Entrance Entrance (I)": {
        'type': 'interior',
        'link': {
            "Sahasrahla's House East Entrance Entrance (E)": [],
            "Sahasrahla's House West Entrance Entrance (I)": []}
    },
    "Sahasrahla's House West Entrance Entrance (I)": {
        'type': 'interior',
        'link': {
            "Sahasrahla's House East Entrance Entrance (I)": [],
            "Sahasrahla's House West Entrance Entrance (E)": []}
    },
    "Sahasrahla's House West Entrance Entrance (E)": {
        'type': 'entrance_unique', 'map': 'light', 'coord': (101, 279),
        'link': {
            "Sahasrahla's House West Entrance Entrance (I)": [],
            'Kakariko': []}
    },
}



'''
Northern Dark World
'''

__all__ = 'LOCATIONS',


LOCATIONS = {
    'North Dark World': {
        'type': 'area',
        'link': {
            'North Light World': [('and', [
                ('nosettings', 'inverted'), ('item', 'mirror')])],
            "Thieves' Town Surface": [],
            'Skull Woods Surface': [],
            "Thieves' Town Fortune Teller Entrance (E)": [],
            'South Specialty Shop Entrance (E)': [],
            'Bumper Cave Bottom Entrance Entrance (E)': [
                ('item', 'powerglove')],
            'Bumper Cave Top': [
                ('glitch', 'major'),
                ('and', [
                    ('glitch', 'overworld'), ('item', 'pegasus')])],
            'North Specialty Shop Entrance (E)': [],
            'Dark Chapel Entrance (E)': [],
            'Haunted Graveyard': [('rabbitbarrier', None)],
            'Dark River': [
                ('item', 'flippers'),
                ('and', [
                    ('glitch', 'overworld'), ('item', 'pegasus')])]},
        'visible': {'Bumper Cave': []}
    },
    "Thieves' Town Fortune Teller Entrance (E)": {
        'type': 'entrance', 'map': 'dark', 'coord': (119, 214),
        'link': {
            'North Dark World': [],
            "Thieves' Town Fortune Teller Entrance (I)": []}
    },
    "Thieves' Town Fortune Teller Entrance (I)": {
        'type': 'interior',
        'link': {
            "Thieves' Town Fortune Teller Entrance (E)": []}
    },
    'South Specialty Shop Entrance (E)': {
        'type': 'entrance_shop', 'map': 'dark', 'coord': (214, 302),
        'link': {
            'North Dark World': [],
            'South Specialty Shop Entrance (I)': []}
    },
    'South Specialty Shop Entrance (I)': {
        'type': 'interior',
        'link': {
            'South Specialty Shop Entrance (E)': []}
    },
    'Bumper Cave Bottom Entrance Entrance (E)': {
        'type': 'entrance_unique', 'map': 'dark', 'coord': (236, 116),
        'link': {
            'North Dark World': [],
            'Bumper Cave Bottom Entrance Entrance (I)': [
                ('nosettings', 'inverted')],
            'Hebra Ascent (Bottom) Entrance (I)': [('settings', 'inverted')]}
    },
    'Bumper Cave Bottom Entrance Entrance (I)': {
        'type': 'interior',
        'link': {
            'Bumper Cave Bottom Entrance Entrance (E)': [
                ('nosettings', 'inverted')],
            'Hebra Ascent (Bottom) Entrance (E)': [('settings', 'inverted')],
            'Bumper Cave 1': []}
    },
    'Bumper Cave 1': {
        'type': 'area',
        'link': {
            'Bumper Cave Bottom Entrance Entrance (I)': [],
            'Bumper Cave 2': [
                ('and', [
                    ('settings', 'placement_advanced'),
                    ('rabbitbarrier', None)]),
                ('item', 'hookshot')]}
    },
    'Bumper Cave 2': {
        'type': 'area',
        'link': {
            'Bumper Cave 1': [],
            'Bumper Cave 3': [('item', 'cape')]}
    },
    'Bumper Cave 3': {
        'type': 'area',
        'link': {
            'Bumper Cave 2': [('item', 'cape')],
            'Bumper Cave Top Entrance Entrance (I)': []}
    },
    'Bumper Cave Top Entrance Entrance (I)': {
        'type': 'interior',
        'link': {
            'Bumper Cave 3': [],
            'Bumper Cave Top Entrance Entrance (E)': [
                ('nosettings', 'inverted')],
            'Hebra Descent (Bottom) Entrance (E)': [('settings', 'inverted')]}
    },
    'Bumper Cave Top Entrance Entrance (E)': {
        'type': 'entrance_unique', 'map': 'dark', 'coord': (238, 101),
        'link': {
            'Bumper Cave Top Entrance Entrance (I)': [
                ('nosettings', 'inverted')],
            'Hebra Descent (Bottom) Entrance (I)': [('settings', 'inverted')],
            'Bumper Cave Top': []}
    },
    'Bumper Cave Top': {
        'type': 'area',
        'link': {
            'Hebra Descent End': [('and', [
                ('nosettings', 'inverted'), ('item', 'mirror')])],
            'Bumper Cave Top Entrance Entrance (E)': [],
            'Bumper Cave': [],
            'North Dark World': []}
    },
    'Bumper Cave': {
        'type': 'item', 'map': 'dark', 'coord': (223, 101),
        'link': {
            'Bumper Cave Top': []}
    },
    'North Specialty Shop Entrance (E)': {
        'type': 'entrance_shop', 'map': 'dark', 'coord': (217, 37),
        'link': {
            'North Dark World': [],
            'North Specialty Shop Entrance (I)': []}
    },
    'North Specialty Shop Entrance (I)': {
        'type': 'interior',
        'link': {
            'North Specialty Shop Entrance (E)': []}
    },
    'Dark Chapel Entrance (E)': {
        'type': 'entrance', 'map': 'dark', 'coord': (300, 181),
        'link': {
            'North Dark World': [],
            'Dark Chapel Entrance (I)': []}
    },
    'Haunted Graveyard': {
        'type': 'area',
        'link': {
            'North Dark World': [('rabbitbarrier', None)],
            'Graveyard Cave Entrance (E)': [('and', [
                ('nosettings', 'inverted'), ('item', 'mirror')])],
            "King's Tomb Entrance (E)": [('and', [
                ('nosettings', 'inverted'), ('item', 'mirror')])]}
    },
    'Dark Chapel Entrance (I)': {
        'type': 'interior',
        'link': {
            'Dark Chapel Entrance (E)': []}
    }
}



'''
TTK style settings
'''

import tkinter.ttk as ttk

from .config import CONFIG, FONT

__all__ = 'init',


def init() -> None:
    '''
    Initialise ttk style.
    '''

    style = ttk.Style()
    style.configure('.', font=FONT)
